<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	PhocaDownload
* @contact 		support+payplans@readybytes.in
* 
* Highly Inspired from AEC micro integration for PhocaDownload
*/
if(defined('_JEXEC')===false) die();

class PayplansAppPhocadownload extends PayplansApp
{
	protected $_location	= __FILE__;
	protected $_resource	= array('access' => 'com_phocadownlaod.category.access',
									'upload' => 'com_phocadownlaod.category.upload');
	
	// applicable only if phocadownload exist
	public function _isApplicable(PayplansIfaceApptriggerable $refObject, $eventName='')
	{
		return JFolder::exists(JPATH_SITE .DS.'components'.DS.'com_phocadownload');
	}
	

	public function onPayplansSubscriptionAfterSave($prev, $new)
	{
		// no need to trigger if previous and current state is same
		if($prev != null && $prev->getStatus() == $new->getStatus()){
			return true;
		}

		$newStatus  	= $new->getStatus();
		$prevStatus 	= ($prev != null) ? $prev->getStatus() : PayplansStatus::NONE; 
		
		$accessOnActive = $this->getAppParam('accessToCatOnActive');
		$accessOnActive	= (is_array($accessOnActive)) ? $accessOnActive : array($accessOnActive);
		
		$accessOnHold 	= $this->getAppParam('accessToCatOnHold');
		$accessOnHold	= (is_array($accessOnHold)) ? $accessOnHold : array($accessOnHold);
		
		$accessOnExpire = $this->getAppParam('accessToCatOnExpire');
		$accessOnExpire	= (is_array($accessOnExpire)) ? $accessOnExpire : array($accessOnExpire);
			
		$uploadOnActive = $this->getAppParam('uploadToCatOnActive');
		$uploadOnActive	= (is_array($uploadOnActive)) ? $uploadOnActive : array($uploadOnActive);
		
		$uploadOnHold 	= $this->getAppParam('uploadToCatOnHold');
		$uploadOnHold	= (is_array($uploadOnHold)) ? $uploadOnHold : array($uploadOnHold);
		
		$uploadOnExpire = $this->getAppParam('uploadToCatOnExpire');
		$uploadOnExpire	= (is_array($uploadOnExpire)) ? $uploadOnExpire : array($uploadOnExpire);
		
		$subid 	 		= $new->getId();
		$userid 		= $new->getBuyer();
		 
		// #1 : remove from groups assigned from other status (if applicable)
		// #2 : add to group and add resource for current status 
		
		// if subscription is active
		if($newStatus == PayplansStatus::SUBSCRIPTION_ACTIVE){
			$this->_removeFromCat($userid, $accessOnHold, $subid);
			$this->_removeFromCat($userid, $accessOnExpire, $subid);
			$this->_addToCat($userid, $accessOnActive, $subid);
			
			$this->_removeFromCat($userid, $uploadOnHold, $subid, 'upload');
			$this->_removeFromCat($userid, $uploadOnExpire, $subid, 'upload');
			$this->_addToCat($userid, $uploadOnActive, $subid, 'upload');
			return true;
		}
		
		if($newStatus == PayplansStatus::SUBSCRIPTION_HOLD){
			$this->_removeFromCat($userid, $accessOnActive, $subid);
			$this->_removeFromCat($userid, $accessOnExpire, $subid);
			$this->_addToCat($userid, $accessOnHold, $subid);
			
			$this->_removeFromCat($userid, $uploadOnActive, $subid, 'upload');
			$this->_removeFromCat($userid, $uploadOnExpire, $subid, 'upload');
			$this->_addToCat($userid, $uploadOnHold, $subid, 'upload');
			return true;
		}
		
		if($newStatus == PayplansStatus::SUBSCRIPTION_EXPIRED){
			$this->_removeFromCat($userid, $accessOnActive, $subid);
			$this->_removeFromCat($userid, $accessOnHold, $subid);
			$this->_addToCat($userid, $accessOnExpire, $subid);
			
			$this->_removeFromCat($userid, $uploadOnActive, $subid, 'upload');
			$this->_removeFromCat($userid, $uploadOnHold, $subid, 'upload');
			$this->_addToCat($userid, $uploadOnExpire, $subid, 'upload');
			return true;
		}
	
		return true;
	}


	protected function _addToCat( $userid, $catids, $subid, $for='access')
	{
		if(!is_array($catids)){
			return false;
		}
		
		foreach($catids as $catid){
			if(empty($catid)){
				continue;
			}
			
			$users = $this->_getCatMembers($catid, $for);
	
			if(in_array( $userid, $users ) ) {
				$this->_addToResource($subid, $userid, $catid, $this->_resource[$for]);
				continue;
			} 
			
			// Make sure we have no empty value
			$search = 0;
			while ( $search !== false ) {
				$search = array_search( '', $users );
				if( $search !== false ){
					unset( $users[$search] );
				}
			}
	
			$users[] = $userid;
			
			if($this->_setCatMembers($catid, $users, $for)){
				$this->_addToResource($subid, $userid, $catid, $this->_resource[$for]);
			}
		}
		
		return true;
	}

	protected function _removeFromCat($userid, $catids, $subid, $for='access')
	{
		if(!is_array($catids)){
			return false;
		}
		
		foreach($catids as $catid){
			if(empty($catid)){
				continue;
			}
			
			$users = $this->_getCatMembers($catid, $for);
	
			if(!in_array($userid,$users)){
				continue;
			}
			
			$key = array_search( $userid, $users );
			unset( $users[$key] );
	
			// Make sure we have no empty value
			$search = 0;
			while ( $search !== false ) {
				$search = array_search( '', $users );
				if ( $search !== false ) {
					unset( $users[$search] );
				}
			}
			
			$this->_removeFromResource($subid, $userid, $catid, $this->_resource[$for]); 
			$this->_setCatMembers($catid, $users, $for);
		}
		return true;
	}
	
	protected function _getCatMembers($catid, $for)
	{
		$db = JFactory::getDBO();

		$query = 'SELECT `accessuserid`'
			. ' FROM #__phocadownload_categories'
			. ' WHERE `id` = \'' . $catid . '\''
			;
			
		if($for == 'upload'){
			$query = 'SELECT `uploaduserid`'
			. ' FROM #__phocadownload_categories'
			. ' WHERE `id` = \'' . $catid . '\''
			;
		}
		
		$db->setQuery( $query );
		return explode(',', $db->loadResult());
	}
	
	protected function _setCatMembers($catid, $users, $for)
	{
		$db = JFactory::getDBO();
		$query = 'UPDATE #__phocadownload_categories'
			. ' SET `accessuserid` = \'' . implode( ',', $users ) . '\''
			. ' WHERE `id` = \'' . $catid . '\''
			;
			
		if($for == 'upload'){
			$query = 'UPDATE #__phocadownload_categories'
					. ' SET `uploaduserid` = \'' . implode( ',', $users ) . '\''
					. ' WHERE `id` = \'' . $catid . '\''
					;
		}
		$db->setQuery( $query );
		return $db->query();
	}
	
	public static function getCategories()
	{
		static $categories = null;
		
		if (!empty($categories) )
		{
			return $categories;	
		}
		
		$db = JFactory::getDBO();
		$query = 'SELECT id, title, description, parent_id'
			 	. ' FROM #__phocadownload_categories'
			 	;
			 	
	 	$db->setQuery( $query );
	 	
	 	$categories = $db->loadObjectList('id');

	 	return $categories; 
	} 
	
	//render Widget
	public function renderWidgetHtml()
	{   
		//get user id
		$userid           = XiFactory::getUser()->id;
		if(!$userid){
			return '';
		}
		//get user's phoca categories
	    $accessphoca_categories = self::_getUserAccessPhocaCategories($userid);
	    $uploadphoca_categories = $this->_getUserUploadPhocaCategories($userid);
	    if(empty($accessphoca_categories) && empty($uploadphoca_categories))
	    {
	    	return '';
	    }
	    $this->assign('accessphoca_categories',$accessphoca_categories);
	    $this->assign( 'uploadphoca_categories',  $uploadphoca_categories);
        $data             = $this->_render('widgethtml');
        return $data;
	}	
	
	public static function _getUserAccessPhocaCategories($userid)
	{
		$db            = XiFactory::getDBO();
		$phoca_categories = array();
		$query         = 'SELECT `id`, `title`, `accessuserid`'
			 	         . ' FROM #__phocadownload_categories';
		$db->setQuery($query);
		$all_categories = $db->loadObjectList('id');
	    if (count($all_categories)) {
            foreach ($all_categories as $category) {
                $accessUsersList     = array();
                $accessUsersList     = explode(',', $category->accessuserid);
                if (in_array($userid , $accessUsersList)){
				  	$phoca_categories[$category->id] = $category->title;
                }
            }
        }
        return $phoca_categories;
	}
	
	protected function _getUserUploadPhocaCategories($userid)
	{
		$db            = XiFactory::getDBO();
		$phoca_categories = array();
		$query         = 'SELECT `id`,`title`, `uploaduserid`'
			 	         . ' FROM #__phocadownload_categories';
		$db->setQuery($query);
		$all_categories = $db->loadObjectList('id');
	    if (count($all_categories)) {
            foreach ($all_categories as $category) {
                $uploadUsersList     = array();
                $uploadUsersList     = explode(',', $category->uploaduserid );
                if (in_array($userid , $uploadUsersList)){
				  	$phoca_categories[$category->id] = $category->title;
                }
            }
        }
        return $phoca_categories;
	}
	
    function getNameFromResourceValue($resource, $value)
	{
		// if its a different resource
		if(!in_array($resource,$this->_resource)){
			return false;
		}
		$categories = $this->getCategories();
		return $categories[$value]->title;
		
	}
}

